/**  @file zdsproj_Launcher.c

     Project Launch Steering Utility for
     ZiLOG Developer Studio, which opens
     project files of type 'zdsproj'.

The problem with ZiLOG Tools is they have at least
three different flavors of tool, and if you like
Zilog parts enough to use more than one tool chain,
your Windows file association is reset to the last
installed (most recently) tool path.  This small
utility overcomes the limitation somewhat.  It still
requires manually setting the path in the .INI file,
and re-associating this utility with the 'zdsproj'
file type after a new ZDS tool install.  The benefit
for ease-of-use is well worth the small amount of
effort to install and configure it.

Now the user can double-click any '.zdsproj' file
and the utility will invoke the proper tool chain.

The tool in its current state supports 3 tool chains;
eZ80/Acclaim Family, Z8-Encore (Z8F Family), and the
ZNEO (Z16F Family).

<BR> Copyright(C) 2007 Douglas Beattie Jr
<P>
<BR> Author: Douglas Beattie Jr
<BR> Created: DBJR 12/31/2007 7:41:56 PM

     @note
	Last change: DBJR 1/1/2008 1:02:27 PM
*/
/* $Log: C:\qVCS\Archives\util\zds2_Launcher\zdsproj_Launcher.d $
 * 
 *   zdsproj_Launcher.c
 * 
 * Revision 1.0  by: Douglas Beattie Jr.  Rev date: 1/1/2008 1:13:48 PM
 *   Initial Revision
 * 
 * $Endlog$
 */

#include <stdio.h>
#include <string.h>
#include <windows.h>
#include <conio.h>
#include "inifile.h"
#include "zdsproj_Launcher.h"


static int project_type;

static FILE *inpfile;
static char inpbufr[MAX_BUF];

static char zds_cmdline[MAX_BUF] = "\0";
static char arg0_path[MAX_BUF];

static char *get_arg_path(const char * arg_str);
static int tool_exec(const char *arg1);


int main(int argc, char *argv[])
{
     char * sptr;

#ifndef NDEBUG
     printf("EX'D: [%s]\n",argv[0]);
     printf("PATH: [%s]\n",get_arg_path(argv[0]));
#endif

     if (argc > 1) {

          if((inpfile = fopen(argv[1],"r")) != NULL) {
#ifndef NDEBUG
               printf("Opened input file %s\n",argv[1]);
#endif
          }
          else {
               fprintf(stderr,"Error opening file ''%s''\n",argv[1]);
               return -2;
          }
     }
     else {
          fprintf(stderr,"No file specified...\n");
          return -1;
     }



     /****************************************/
     if ( inifile_main(get_arg_path(argv[0])) != 0 ) {
          fprintf(stderr,"PROBLEM READING .INI FILE, '%s'.\n",INI_FILE_NAME);
          return -5;
     }

     /****************************************/
     project_type = PROJ_TYPE_UNKNOWN; //initialize

     do {
          if (fgets(inpbufr,255,inpfile) != NULL) {
               //printf("OLD: %s\n",inpbufr);

               // Find <cpu> tag in ZDS2 project file
               sptr = strstr(inpbufr,"<cpu>EZ80");
               if  (sptr != NULL) {
                    project_type = PROJ_TYPE_EZ80;
                    break;     
               }
               sptr = strstr(inpbufr,"<cpu>Z8F");
               if  (sptr != NULL) {
                    project_type = PROJ_TYPE_Z8F;
                    break;     
               }
               sptr = strstr(inpbufr,"<cpu>Z16F");
               if  (sptr != NULL) {
                    project_type = PROJ_TYPE_Z16F;
                    break;     
               }
          }
     } while (! feof(inpfile));

     fclose(inpfile);

     if (project_type != PROJ_TYPE_UNKNOWN) {
          //invoke the proper application
          return tool_exec((const char *)argv[1]);
     }

	return 1;
}   //lint !e818


/**  Attempt to invoke the proper application.
 */
int tool_exec(const char *arg1)
{
     const char no_id_str[] = "TOOL NOT IDENTIFIED, CAN'T LAUNCH PROJECT\n";

     STARTUPINFO si;
     PROCESS_INFORMATION pi;

     ZeroMemory( &si, sizeof(si) );
     si.cb = sizeof(si);

     switch (project_type) {

          case PROJ_TYPE_EZ80:
               if (!strlen(ez80_cmdline)) {
                    printf("%s",no_id_str);
                    (void) getch();
                    return -6;
               }
               strcpy(zds_cmdline,ez80_cmdline);
               break;

          case PROJ_TYPE_Z8F:
               if (!strlen(z8f_cmdline)) {
                    printf("%s",no_id_str);
                    (void) getch();
                    return -6;
               }
               strcpy(zds_cmdline,z8f_cmdline);
               break;

          case PROJ_TYPE_Z16F:
               if (!strlen(z16f_cmdline)) {
                    printf("%s",no_id_str);
                    (void) getch();
                    return -6;
               }
               strcpy(zds_cmdline,z16f_cmdline);
               break;

          default:
               return -4; // should never get here
          }

     strcat(zds_cmdline," \"");
     strcat(zds_cmdline,arg1);
     strcat(zds_cmdline,"\"");

#ifndef NDEBUG
     printf("\nCMD: [%s]\n",zds_cmdline);
     return 0; // return here if testing
#endif

     // expect a warning here for "unreachable code"
     // unless NDEBUG is defined

     if( !CreateProcess( NULL, // No module name (use command line).
               zds_cmdline,
               NULL,          // Process handle not inheritable.
               NULL,          // Thread handle not inheritable.
               FALSE,         // Set handle inheritance to FALSE.
               0,             // No creation flags.
               NULL,          // Use parent's environment block.
               NULL,          // Use parent's starting directory.
               &si,           // Pointer to STARTUPINFO structure.
               &pi )          // Pointer to PROCESS_INFORMATION structure.
          )  {
          return -3; // was a problem...
     }

     return 0;
}

/**  Strip the name of executable from arg[0] path.
     The path of the executable must be stripped of
     the executable file name, passed to the INI
     file reader, and then the INI file name appended
     to create the path and filespec.
*/
char *get_arg_path(const char * arg_str)
{
     char * sptr;

#ifndef NDEBUG
     printf("ENTER 'get_arg_path()'\n");
     printf("arg_str: [%s]\n",arg_str);
#endif
     strcpy(arg0_path,arg_str);
#ifndef NDEBUG
     printf("arg0_path: [%s]\n",arg0_path);
#endif
     sptr = &arg0_path[0] + strlen(arg0_path);

#ifndef NDEBUG
     printf("BEGIN: {");
#endif
     while ( sptr >  &arg0_path[0]) {
#ifndef NDEBUG
          (void) putch(*sptr);
#endif
          if ( *--sptr == '\\' ) {
               *++sptr = '\0';
               break;
          }
     }
#ifndef NDEBUG
     printf("} END\n");
#endif

     return arg0_path;
}
